﻿using UnityEngine;
using UnityEngine.AI;

public class Enemy : Character {
	public enum enemyType {
		Basic,
		Armored,
		Fast
	}

	public enemyType type = enemyType.Basic;
	public bool cooperating = false;
	public static bool cooperatingVisibility = false;
	
	private GameObject player;
	NavMeshAgent navMeshAgent;

	private int wallLayer;
	private bool playerVisible;
	public GameObject[] waypoints;
	private int currentWaypoint;

	private double nextAttackTime;
	private double attackPeriod;

	protected new void Start() {
		base.Start();

		player = GameObject.FindGameObjectWithTag("Player");
		setUpType();
		setUpNavMeshAgent();

		wallLayer = 1 << LayerMask.NameToLayer("Walls");
		currentWaypoint = waypoints.Length > 0 ? 0 : -1;
		
		attackPeriod = 1 / (PlayerPrefsManager.GetDifficulty() / 7 + .334);
	}

	private void setUpNavMeshAgent() {
		navMeshAgent = GetComponent<NavMeshAgent>();
		navMeshAgent.speed = movementSpeed;
		navMeshAgent.stoppingDistance = (GetComponent<CapsuleCollider>().radius + player.GetComponent<CapsuleCollider>().radius) * 1.1f;
	}

	private void setUpType() {
		switch (type) {
			case enemyType.Basic:
				break;
			case enemyType.Armored:
				health = 100;
				movementSpeed *= .8f;
				break;
			case enemyType.Fast:
				health = 30;
				movementSpeed *= 1.5f;
				break;
			default:
				Debug.LogError("Enemy type not defined");
				break;
		}
	}

	private void Update() {
		CheckWaypoint();
		SetDestination();
		Attack();
	}

	private void CheckWaypoint() {
		if (currentWaypoint >= 0) {
			bool currentWaypointChecked = Vector3.Distance(transform.position, waypoints[currentWaypoint].transform.position) <= navMeshAgent.stoppingDistance * 1.1f;
			if (currentWaypointChecked) {
				if (currentWaypoint + 1 == waypoints.Length) {
					currentWaypoint = 0;
				} else {
					currentWaypoint++;
				}
			}
		}
	}

	private void SetDestination() {
		playerVisible = player != null ? !Physics.Linecast(transform.position, player.transform.position, wallLayer) : false;
		Vector3 destination;
		if (playerVisible && cooperating) {
			cooperatingVisibility = true;
			Invoke("ResetCooperatingVisibility", 3f);
		}
		if ((playerVisible || (cooperatingVisibility && cooperating)) && player) {
			destination = player.transform.position;
		} else if (currentWaypoint >= 0) {
			destination = waypoints[currentWaypoint].transform.position;
		} else {
			destination = transform.position;
		}
		navMeshAgent.destination = destination;
	}

	private new void Attack() {
		if (Time.time > nextAttackTime) {
			nextAttackTime += attackPeriod;
			if (player && Vector3.Distance(transform.position, player.transform.position) < 1.5f * navMeshAgent.stoppingDistance) {
				base.Attack();
			}
		}
	}

	private void ResetCooperatingVisibility() {
		cooperatingVisibility = false;
	}
}
