﻿using UnityEngine;
using UnityEngine.SceneManagement;

public class MusicManager : MonoBehaviour {
	public AudioClip[] levelMusicArray;
	private AudioSource currentPlaying;

	static MusicManager instance = null;

	void Awake() {
		InstanceSetup();
		
		currentPlaying = GetComponent<AudioSource>();
		currentPlaying.enabled = true;
		currentPlaying.volume = PlayerPrefsManager.GetMusicVolume();
	}

	private void InstanceSetup() {
		if (instance != null) {
			Destroy(gameObject);
		} else {
			instance = this;
			DontDestroyOnLoad(gameObject);
		}
	}

	void OnEnable() {
		SceneManager.sceneLoaded += OnSceneLoaded;
	}

	void OnSceneLoaded(Scene scene, LoadSceneMode mode) {
		int level = scene.buildIndex;
		if (levelMusicArray.Length > level) {
			AudioClip newAudioClip = levelMusicArray[level];

			if (currentPlaying.isPlaying && newAudioClip) {
				if (currentPlaying.clip != newAudioClip) {
					currentPlaying.Stop();
				}
			}
			if (newAudioClip && currentPlaying.clip != newAudioClip) {
				currentPlaying.clip = newAudioClip;
				currentPlaying.loop = true;
				if (scene.name.ToString().Contains("03")) {
					currentPlaying.loop = false;
				}
				currentPlaying.Play();
			}
		} else {
			Debug.LogWarning(name + " - levelMusicArray is to short. Given scene can not has background music defined");
		}
	}
}
