﻿using UnityEngine;
using UnityEngine.SceneManagement;

public static class PlayerPrefsManager {
	public const string DIFFICULTY_KEY = "difficulty";
	public const string MUSIC_VOLUME_KEY = "music_volume";
	public const string SOUND_EFFECTS_VOLUME_KEY = "sound_effects_volume";
	public const string GAME_COMPLETED = "game_completed";
	const string LEVEL_KEY = "level_unlocked_";
	const string LAST_UNLOCKED_LEVEL = "last_level_unlocked";

	public const float DIFFICULTY_DEFAULT = 2f;
	public const float MUSIC_VOLUME_DEFAULT = 0.8f;
	public const float SOUND_EFFECTS_VOLUME_DEFAULT = 0.8f;

	public static void SetDifficulty(float difficulty) {
		if (difficulty >= 1f && difficulty <= 3f) {
			PlayerPrefs.SetFloat(DIFFICULTY_KEY, difficulty);
		} else {
			Debug.LogError("Difficulty value out of range");
		}
	}

	public static float GetDifficulty() {
		return PlayerPrefs.GetFloat(DIFFICULTY_KEY);
	}

	public static void SetMusicVolume(float volume) {
		if (volume >= 0f && volume <= 1f) {
			PlayerPrefs.SetFloat(MUSIC_VOLUME_KEY, volume);
		} else {
			Debug.LogError("Music volume value out of range");
		}
	}

	public static float GetMusicVolume() {
		return PlayerPrefs.GetFloat(MUSIC_VOLUME_KEY);
	}

	public static void SetSoundEffectsVolume(float volume) {
		if (volume >= 0f && volume <= 1f) {
			PlayerPrefs.SetFloat(SOUND_EFFECTS_VOLUME_KEY, volume);
		} else {
			Debug.LogError("Sound effects volume value out of range");
		}
	}

	public static float GetSoundEffectsVolume() {
		return PlayerPrefs.GetFloat(SOUND_EFFECTS_VOLUME_KEY);
	}

	public static void SetPlayerPrefsBool(string flagName, bool setTrue) {
		PlayerPrefs.SetInt(flagName, setTrue ? 1 : 0);
	}

	public static void SetPlayerPrefsInt(string flagName, int value) {
		PlayerPrefs.SetInt(flagName, value);
	}

	public static void UnlockLevel(int level) {
		if (level >= 0 && level <= SceneManager.sceneCountInBuildSettings - 1) {
			SetPlayerPrefsInt(LEVEL_KEY + level.ToString(), 1);
			PlayerPrefs.DeleteKey(LAST_UNLOCKED_LEVEL); // TODO: move this somewhere...
			if (level != 1) {
				SetPlayerPrefsInt(LAST_UNLOCKED_LEVEL, level);
			}
		}
	}

	public static bool IsLevelUnlocked(int level) {
		int? levelUnlocked = PlayerPrefs.GetInt(LEVEL_KEY + level.ToString());
		return levelUnlocked != null && levelUnlocked == 1;
	}

	public static void SetLastLevelUnlocked(int level) {
		PlayerPrefs.SetInt(LAST_UNLOCKED_LEVEL, level);
	}

	public static int? GetLastLevelUnlocked() {
		return PlayerPrefs.GetInt(LAST_UNLOCKED_LEVEL);
	}
}