﻿using UnityEngine;
using UnityEngine.UI;

public class LevelSelectionController : MonoBehaviour {
	public GameManager gameManager;
	public Button LevelButtonPrefab;
	private int levelTotalCount;
	private float spaceMultipler = 1.3f;

	void Start() {
		levelTotalCount = GameManager.levelNumber;

		GameObject levelTilesParent = GameObject.Find("LevelTiles");
		int tilesInRow = (int) Mathf.Floor(levelTilesParent.GetComponent<RectTransform>().rect.width / (LevelButtonPrefab.GetComponent<RectTransform>().rect.width * spaceMultipler));
		int tilesPlacedInRow = 0;
		int columnCounter = 0;

		for (int levelNumber = 1; levelNumber <= levelTotalCount; levelNumber++) {
			Button levelButton = Instantiate(LevelButtonPrefab);

			SetLevelButtonName(levelButton, levelNumber);
			SetLevelButtonBehaviour(levelButton, levelNumber);
			SetLevelButtonPosition(levelButton, levelTilesParent, tilesPlacedInRow, columnCounter);
			
			if (++tilesPlacedInRow >= tilesInRow) {
				tilesPlacedInRow = 0;
				columnCounter++;
			}
		}
	}

	private void SetLevelButtonName(Button levelButton, int levelNumber) {
		string levelNumberAsString = levelNumber < 10 ? "0" + levelNumber.ToString() : levelNumber.ToString();
		levelButton.name = "Level_" + levelNumberAsString;
		levelButton.GetComponentInChildren<Text>().text = levelNumberAsString;
	}

	private void SetLevelButtonBehaviour(Button levelButton, int levelNumber) {
		if (PlayerPrefsManager.IsLevelUnlocked(levelNumber)) {
			levelButton.interactable = true;
			levelButton.onClick.AddListener(delegate { OnClickLoadLevel(levelNumber); });
		} else {
			levelButton.interactable = false;
		}
	}

	private void SetLevelButtonPosition(Button levelButton, GameObject levelTilesParent, int tilesPlacedInRow, int columnCounter) {
		RectTransform levelButtonRectTransform = levelButton.GetComponent<RectTransform>();
		levelButton.transform.SetParent(levelTilesParent.transform);
		float xPosition = 1 / 2 * levelButtonRectTransform.rect.width + tilesPlacedInRow * (levelButtonRectTransform.rect.width * spaceMultipler);
		float yPosition = 1 / 2 * levelButtonRectTransform.rect.height + columnCounter * (levelButtonRectTransform.rect.height * spaceMultipler);
		if (yPosition + levelButtonRectTransform.rect.height > levelTilesParent.GetComponent<RectTransform>().rect.height) {
			Debug.LogError("Level tiles doesn't fit");
		}

		levelButtonRectTransform.SetInsetAndSizeFromParentEdge(RectTransform.Edge.Left, xPosition, levelButtonRectTransform.rect.width);
		levelButtonRectTransform.SetInsetAndSizeFromParentEdge(RectTransform.Edge.Top, yPosition, levelButtonRectTransform.rect.height);
	}

	private void OnClickLoadLevel(int levelNumber) {
		PlayerPrefsManager.SetLastLevelUnlocked(levelNumber);
		gameManager.LoadLevel("02 Level_" + GetLevelNumberAsString(levelNumber));
	}

	private string GetLevelNumberAsString(int levelNumber) {
		return levelNumber < 10 ? "0" + levelNumber.ToString() : levelNumber.ToString();
	}
}